/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   welcome.c
Author(s)   :   Henry Fok, Thomas Cheng
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   welcome screen and low battery screen
Revision    :   1.1
Note        :   None
===========================================================================
*/              



#include "pr31700s.h"
#include "pr31700c.h"
#include "intc.h"
#include "uart.h"
#include "print.h"
#include "tmrapi.h"
#include "msg.h"
#include "eventmgr.h"
#include "mmu.h"
#include "alaunch.h"
#include "ttp.h"
#include "lcdapi.h"
#include "dev_pwr.h"

//#define DEBUG

extern EvtQueueType evtQueue;
extern MsgQueueType	msgQueue;

UWORD lowbattmr, weltmr;

const UWORD lowbat[500] = {
0x0fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
0xffffffff, 0xff0f0000, 0x30000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00cf0000, 0xcaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaa00000, 0x0002aaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000,
0xca855555, 0x5557a8a8, 0xa8aaaaaa, 0x8aaaaaaa, 0xa80aaa8a, 0x8aaaaaaa, 0xa2aaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaabf0000, 0xca855540, 0x5557a8a8, 0xa8aaaaaa, 0xaaaaaaaa, 0xa8a2aa8a, 0x8aaaaaaa,
0xa2aaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaa15547, 0x555ea8a8, 0xa8828882, 0x882a2aaa,
0xa8a20a02, 0x028a2222, 0xa2aa2888, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaa15547, 0x555eaa22,
0x22a88288, 0x88888a80, 0xa80aa28a, 0x8a220a22, 0xa2a88888, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000,
0xcaa85547, 0x557aaa22, 0x22808a88, 0x88888aaa, 0xa8a2028a, 0x8a022a22, 0xa2a88888, 0xaaaaaaaa,
0xaaaaaaaa, 0xaabf0000, 0xcaa85547, 0x557aaa8a, 0x8a888a88, 0x888a0aaa, 0xa8a2228a, 0x8a2a2a82,
0xa2a88a22, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaa1547, 0x55eaaa8a, 0x8a808a88, 0x888a8aaa,
0xa80a02a2, 0xa2822aa2, 0xa00a2a22, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaa1547, 0x55eaaaaa,
0xaaaaaaaa, 0xaaa82aaa, 0xaaaaaaaa, 0xaaaaaa0a, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000,
0xcaaa857f, 0x57aaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaabf0000, 0xcaaa8555, 0x57aaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaa143, 0x5eaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaa147, 0x5eaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 
0xcaaaa87f, 0x7aaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaabf0000, 0xcaaaa855, 0x7aaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaaa15, 0xeaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa,
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaaa15, 0xeaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 
0xcaaaaa87, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaabf0000, 0xcaaaaa8f, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 0xcaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000, 
0xcaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaabf0000, 0xcbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xe2bf0000, 0xcb000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 
0xcb000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000,
0xcb000000, 0x00000000, 0x00000000, 0xc0000000, 0x3f000303, 0x00000000, 0xccc00000, 0x00000000, 
0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0xc0000000, 0x30c00303, 0x00000000,
0xccc00000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0xc00c3330,
0x30cf0fcf, 0xc30cccc0, 0xccc00000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 
0x00000000, 0xc0333330, 0x3f00c303, 0x0ccf0cc0, 0xccc00000, 0x00000000, 0x00000000, 0x22bf0000,
0xcb000000, 0x00000000, 0x00000000, 0xc0333330, 0x30cfc303, 0x0fcc0cc0, 0xccc00000, 0x00000000, 
0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0xc0330cc0, 0x30ccc303, 0x0c0c03c0, 
0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0xff0c0cc0, 
0x3f0fc0c0, 0xc3cc00c0, 0xccc00000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000f00, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 
0xcb000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000, 0xcb000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x22bf0000,
0xcb000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x22bf0000, 0xcbaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xa2bf0000, 0xc8000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x02bf0000, 0xcaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaabf0000,
0x3aaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 0xaaaaaaaa, 
0xaaaaaaaa, 0xaacf0000, 0x0fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
0xffffffff, 0xffffffff, 0xffffffff, 0xff0f0000, };

const UWORD welcome[1600] = { 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x003ff000, 0x00fc0000, 0x03fc0000, 0x000003fc, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x003ff000, 0x00fc0000, 0x03fc0000, 0x000003fc, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x000ff000, 0x03fc0000, 
0x03f00000, 0x000003fc, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x000ffc00, 0x03fc0000, 0x03f00000, 0x000003fc, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x000ffc00, 0x03f00000, 0x0ff00000, 0x000000fc, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0003fc00, 0x03f00000, 0x0ff00000, 0x000000fc, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0003ff00, 0x0ff00c00, 
0x0fc00000, 0x000000fc, 0x0000fc00, 0x00000300, 0x00000003, 0x00000000, 0x000003c0, 0x00000000, 
0x0003ff00, 0x0ff00f00, 0x0fc00000, 0x000000fc, 0x0003f000, 0x00003c00, 0x00000003, 0x00000000, 
0x00003f00, 0x00000000, 0x0000ff00, 0x0fc00f00, 0x3fc00000, 0x03fffffc, 0x000fc000, 0xfc00fc00, 
0x3fc00003, 0x003fc00f, 0xf000ff00, 0x3ffc0000, 0x0000ffc0, 0x0fc00f00, 0x3fc00000, 0x0ffffffc, 
0x003fc00f, 0xfc03f000, 0xfff00003, 0x003fc00f, 0xfc00fc00, 0xfffc0000, 0x0000ffc0, 0x0fc03fc0, 
0x3f000000, 0x000003fc, 0x00ffc000, 0x0003f000, 0x3ffc0003, 0x0000c000, 0x3c00fc00, 0x00000000, 
0x00003fc0, 0x0fc03fc0, 0x0f0000c0, 0x000000fc, 0x00ffc000, 0x000ff000, 0x0ffc0003, 0x0000c000, 
0x0c00fc00, 0x00000000, 0x00003ff0, 0x03003fc0, 0x0f0000c0, 0x000000fc, 0x00ffc000, 0x000ff003, 
0xfffc0003, 0x0000c000, 0x0c00fc00, 0x00000000, 0x00003ff0, 0x03003ff0, 0x0c0000c0, 0x000000fc, 
0x00ffc000, 0x03fff003, 0xfffc00f3, 0x0000c000, 0x0c00fc00, 0x00000000, 0x00000ff0, 0x03003ff0, 
0x0c0000c0, 0x000000fc, 0x00ffc003, 0xfffff003, 0xfffc00ff, 0x0000c000, 0x0c00fc00, 0x00000000, 
0x00000ffc, 0x03000ff0, 0x000000c0, 0x0ffffffc, 0x00ffc00f, 0xfffff003, 0xfffc03ff, 0x0000c000, 
0x0c00fc00, 0xfffc0000, 0x00000ffc, 0x00000ff0, 0x000000f0, 0x0ffffffc, 0x00ffc000, 0xfffff000, 
0xfff003ff, 0x003cc000, 0x0c00fc00, 0x3ffc0000, 0x000003fc, 0x00000ffc, 0x000000f0, 0x000003fc, 
0x00ffc000, 0xfffffc00, 0x3fc003ff, 0x003fc000, 0x0c00fc00, 0x00000000, 0x000003ff, 0x00000ffc, 
0x0000003c, 0x000000fc, 0x00fff000, 0x0000fc00, 0x00000fff, 0x003fc000, 0x0c00ff00, 0x00000000, 
0x000003ff, 0x000003fc, 0x0000003c, 0x000000fc, 0x00fffc00, 0x0000ff00, 0x00000fff, 0x003fc000, 
0x0c00ff00, 0x00000000, 0x000000ff, 0x000003ff, 0x0000000f, 0x000000fc, 0x00ffff00, 0x0000ffc0, 
0x00003fff, 0x003fc00c, 0x0c00fff0, 0x00000000, 0x000000ff, 0xc00003ff, 0x0000000f, 0xf00000fc, 
0x00fffff0, 0x0000fffc, 0x000fffff, 0x003fc00f, 0xcc00ffff, 0x00000000, 0x000000ff, 0xfc0000ff, 
0xf0000003, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffc3fff, 0xfff00000, 
0x0000003f, 0xfc0000ff, 0xf0000000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xfffc03ff, 0xfff00000, 0x00000000, 0x00000000, 0x00000000, 0x000003ff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffc00000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000fff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfff00000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00003fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x0000ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffc000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x000fffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xfffff000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x003fffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffc00, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffff00, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffc0, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x03ffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0fffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffffc, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x3ff003ff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0xffc03fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xc0000000, 0x00000000, 0x00000000, 0x00000003, 0xffc03fff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xf0000000, 0x00000000, 0x00000000, 0x0000000f, 0xff00ffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf0000000, 0x00000000, 0x00000000, 
0x0000000f, 0xfc00ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfc000000, 
0x00000000, 0x00000000, 0x0000003f, 0xfc03ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xff000000, 0x00000000, 0x00000000, 0x000000ff, 0xf00fffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xff000000, 0x00000000, 0x00000000, 0x000000ff, 0xf00fffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffc00000, 0x00000000, 0x00000000, 
0x000003ff, 0xc03fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfff00000, 
0x00000000, 0x00000000, 0x00000fc0, 0x000000ff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xfff00000, 0x00000000, 0x00000000, 0x00000fff, 0x03ffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfffc0000, 0x00000000, 0x00000000, 0x00003ffc, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffc0000, 0x00000000, 0x00000000, 
0x00003ff0, 0x0fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000, 
0x00000000, 0x00000000, 0x0000ffc0, 0x3fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffff0000, 0x00000000, 0x00000000, 0x0000ffc0, 0x3fffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffc000, 0x00000000, 0x00000000, 0x0003ff00, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffc000, 0x00000000, 0x00000000, 
0x0003fc00, 0xfffffff0, 0x03ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffff000, 
0x00000000, 0x00000000, 0x0003fc03, 0xffffff03, 0xf0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xfffff000, 0x00000000, 0x00000000, 0x000ff00f, 0xfffffc0f, 0xfcffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfffff000, 0x00000000, 0x00000000, 0x000fc00f, 0xfffff03f, 
0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffc00, 0x00000000, 0x00000000, 
0x003f003f, 0xffffc0ff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffc00, 
0x00000000, 0x00000000, 0x003f00ff, 0xffff03ff, 0xf0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffff00, 0x00000000, 0x00000000, 0x003c00ff, 0xfffc0fff, 0xf0ffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffff00, 0x00000000, 0x00000000, 0x00f003ff, 0xfff00fff, 
0xf0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 
0x00fc0fff, 0xffc03fff, 0xc0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 
0x00000000, 0x00000000, 0x00300fff, 0xff00ffff, 0xc3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 0x00c03fff, 0xfc00ffff, 0xc3ffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 0x00c0ffff, 0xf003ffff, 
0x0fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 
0x0300ffff, 0xf003fffc, 0x3ff3ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffffff0, 
0x00000000, 0x00000000, 0x0f03fffc, 0xc00ffffc, 0x3fcfffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 0x3003fff3, 0xf00ffff0, 0xff3fffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 0x000fffcf, 0xf03fffc3, 
0xfcffffff, 0xfffffff0, 0x00000000, 0x00003fff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 
0x003fff0f, 0xc03fffc3, 0xc3ffffff, 0xfff00000, 0xffffffff, 0xfffc0000, 0x0fffffff, 0xfffffff0, 
0x00000000, 0x00000000, 0x003ffc3f, 0xc0ffff00, 0x3fffffff, 0xc003ffff, 0xffffffff, 0xffffffff, 
0x000fffff, 0xfffffff0, 0x00000000, 0x00000000, 0x00fff0ff, 0xc0fffc3f, 0xffffffc0, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xffc00fff, 0xfffffff0, 0x00000000, 0x00000000, 0x00ffc3ff, 0xc0fff0ff, 
0xfffff000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xfffc003f, 0xfffffff0, 0x00000000, 0x00000000, 
0x03ff0fff, 0xc0ffc3ff, 0xfffc0000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xfffc0000, 0xfffffff0, 
0x00000000, 0x00000000, 0x03fc3fff, 0xc0ff3fff, 0xfff00000, 0x3fffffff, 0xffffffff, 0xffffffff, 
0xfffc0000, 0x0ffffff0, 0x00000000, 0x00000000, 0x0ff0ffff, 0xf0f0ffff, 0xff000000, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xffc00000, 0x03fffff0, 0x00000000, 0x00000000, 0x03cfffff, 0xfc0fffff, 
0xfc000000, 0x00003fff, 0xffffffff, 0xffffffff, 0x00000000, 0x00fffff0, 0x00000000, 0x00000000, 
0x003fffff, 0xffffffff, 0xfc000000, 0x00000003, 0xffffffff, 0xffffc000, 0x00000000, 0x003ffff0, 
0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xf0000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x000ffff0, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xf0000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x000ffff0, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 
0xf0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x000ffff0, 0x00000000, 0x00000000, 
0x0fffffff, 0xffffffff, 0xfc000000, 0x00000003, 0xffffffff, 0xffffc000, 0x00000000, 0x003ffff0, 
0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xfc000000, 0x0003ffff, 0xffffffff, 0xffffffff, 
0xc0000000, 0x00fffff0, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xffc00000, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xfff00000, 0x03fffff0, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 
0xfff00000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xfffc0000, 0x3ffffff0, 0x00000000, 0x00000000, 
0x03ffffff, 0xffffffff, 0xffff0000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xfffc0003, 0xfffffff0, 
0x00000000, 0x00000000, 0x03ffffff, 0xffffffff, 0xfffffc00, 0x3fffffff, 0xffffffff, 0xffffffff, 
0xfff0003f, 0xfffffff0, 0x00000000, 0x00000000, 0x03ffffff, 0xffffffff, 0xfffffff0, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xfc003fff, 0xfffffff0, 0x00000000, 0x00000000, 0x03ffffff, 0xffffffff, 
0xffffffff, 0xf003ffff, 0xffffffff, 0xffffffff, 0x003fffff, 0xfffffff0, 0x00000000, 0x00000000, 
0x03ffffff, 0xffffffff, 0xffffffff, 0xfff00000, 0xffffffff, 0xfffc0000, 0x3fffffff, 0xfffffff0, 
0x00000000, 0x00000000, 0x00ffffff, 0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00003fff, 
0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 0x00ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 0x00ffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 
0x00ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffff00, 
0x00000000, 0x00000000, 0x003fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffff00, 0x00000000, 0x00000000, 0x003fffff, 0xffffffff, 0xffc0fc0f, 0xffffc003, 
0xfffffc3f, 0xfffffc0f, 0xfffffc00, 0x3fffff00, 0x00000000, 0x00000000, 0x003fffff, 0xffffffff, 
0xffc0fc0f, 0xffffc0ff, 0xfffffc3f, 0xfffffc0f, 0xfffff03c, 0x0ffffc00, 0x00000000, 0x00000000, 
0x003fffff, 0xffffffff, 0xffc0000f, 0xffffc00f, 0xfffffc3f, 0xfffffc0f, 0xffffc0ff, 0x03fffc00, 
0x00000000, 0x00000000, 0x000fffff, 0xffffffff, 0xffc0fc0f, 0xffffc0ff, 0xfffffc3f, 0xfffffc0f, 
0xffffc0ff, 0x03fffc00, 0x00000000, 0x00000000, 0x000fffff, 0xffffffff, 0xffc0fc0f, 0xffffc0ff, 
0xfffffc00, 0xfffffc0f, 0xfffff03c, 0x0ffff000, 0x00000000, 0x00000000, 0x000fffff, 0xffffffff, 
0xffc0fc0f, 0xffffc003, 0xfffffc00, 0xfffffc0f, 0xfffffc00, 0x3ffff000, 0x00000000, 0x00000000, 
0x0003ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffff000, 
0x00000000, 0x00000000, 0x0003ffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffc000, 0x00000000, 0x00000000, 0x0000ffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffc000, 0x00000000, 0x00000000, 0x0000ffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000, 0x00000000, 0x00000000, 
0x00003fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000, 
0x00000000, 0x00000000, 0x00003fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xfffc0000, 0x00000000, 0x00000000, 0x00000fff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfffc0000, 0x00000000, 0x00000000, 0x00000fff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfff00000, 0x00000000, 0x00000000, 
0x000003ff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffc00000, 
0x00000000, 0x00000000, 0x000000ff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffc00000, 0x00000000, 0x00000000, 0x000000ff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xff000000, 0x00000000, 0x00000000, 0x0000003f, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xff000000, 0x00000000, 0x00000000, 
0x0000000f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfc000000, 
0x00000000, 0x00000000, 0x0000000f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xf0000000, 0x00000000, 0x00000000, 0x00000003, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xc0000000, 0x00000000, 0x00000000, 0x00000000, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x3fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xfffffffc, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xfffffff0, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x03ffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffff00, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x000fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xfffffc00, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0003ffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xfffff000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0000ffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffc000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00003fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffff0000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000fff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xfffc0000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x000000ff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffc00000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0000003f, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xff000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x0000000f, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf0000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xc0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0fffffff, 0xffffffff, 
0xffffffff, 0xfffffffc, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x03ffffff, 0xffffffff, 0xffffffff, 0xffffffc0, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x003fffff, 0xffffffff, 0xffffffff, 0xffffff00, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0003ffff, 0xffffffff, 0xffffffff, 0xfffff000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, };

void WelscrTmr()
{
	TmrIntDisable(weltmr);
	EvtAppendEvt(0, 0, 0, 0, NULL);
}

BOOLEAN Welscr()
{
	BitmapTemplate 		bitmap;
	BOOLEAN				system_battery_empty = FALSE;
	EvtType 			event;

    bitmap.xcoord = 0;
	bitmap.ycoord = 0;
	bitmap.width = 160;
	bitmap.height = 160;
	bitmap.quantisation = 2;

    (const *)bitmap.bitmap_data = welcome;
 	LcdDrawBitmap(&bitmap, COLOR_NO_INVERT);

	MsgQueueCreate(&msgQueue);
	EvtQueueCreate(&evtQueue);

	weltmr = TmrIntEnable(5000, WelscrTmr);

    do
    {
    	if (CheckBatteryEmpty() == 1)
		{
			system_battery_empty = TRUE;
    		break;	
		}

    	EvtGetEvent(&event);
    	
    	if ((event.eventType == POWER_EVENT && event.eventID == BAT_MAIN_EMPTY) ||
    		(event.eventType == POWER_EVENT && event.eventID == BAT_DOOR_OPEN))
    	{
    		system_battery_empty = TRUE;
    		break;    		
    	}   	
#ifdef DEBUG
		printf("\n Welcome - Event = %ld %ld %ld %ld", event.eventType, event.eventID, event.para1, event.para2);
#endif		    	
	}while(event.eventType == POWER_EVENT);    	

	MsgQueueCreate(&msgQueue);
	EvtQueueCreate(&evtQueue);

	TmrIntDisable(weltmr);		
	if (system_battery_empty)
	{
#ifdef DEBUG
		printf("\n Low Bat Screen return FALSE");		
#endif		
		return FALSE;
	}		
	else
	{
#ifdef DEBUG
		printf("\n Low Bat Screen return TRUE");		
#endif			
		return TRUE;		
	}		
}

void LowbatscrTmr()
{
	TmrIntDisable(lowbattmr);
#ifdef DEBUG
	printf("\n Low Bat TimeOut");
#endif		
	EvtAppendEvt(0, 0, 0, 0, NULL);
}

BOOLEAN Lowbatscr()
{
	BitmapTemplate 		bitmap;
	BOOLEAN				system_battery_empty = FALSE;
	EvtType 			event;

    bitmap.xcoord = 5;
	bitmap.ycoord = 47;
	bitmap.width = 150;
	bitmap.height = 50;
	bitmap.quantisation = 2;

    (const *)bitmap.bitmap_data = lowbat;
 	LcdDrawBitmap(&bitmap, COLOR_NO_INVERT);

	MsgQueueCreate(&msgQueue);
	EvtQueueCreate(&evtQueue);

	lowbattmr = TmrIntEnable(5000, LowbatscrTmr);

    do
    {
    	if (CheckBatteryEmpty() == 1)
		{
#ifdef DEBUG
			printf("\n system_battery_empty = TRUE");
#endif						
			system_battery_empty = TRUE;
    		break;	
		}

    	EvtGetEvent(&event);
    	
    	if ((event.eventType == POWER_EVENT && event.eventID == BAT_MAIN_EMPTY) ||
    		(event.eventType == POWER_EVENT && event.eventID == BAT_DOOR_OPEN))
    	{
    		system_battery_empty = TRUE;
    		break;    		
    	}   	
#ifdef DEBUG
		printf("\n LowBatScr - Event = %ld %ld %ld %ld", event.eventType, event.eventID, event.para1, event.para2);
#endif	
    	
	}while(event.eventType == POWER_EVENT);    		


	TmrIntDisable(lowbattmr);
	
#ifdef DEBUG
	printf("\n Quit LowBat Screen - Battery Empty = %ld", system_battery_empty);
#endif	


	
	if (system_battery_empty)
		return FALSE;
	else return TRUE;			
}
